#version 430

layout(binding=0) uniform sampler2D tex0;
layout(binding=1) uniform sampler2D tex1;
layout(binding=2) uniform sampler2D tex2;
layout(binding=3) uniform sampler2D tex3;
layout(binding=4) uniform sampler2D tex4;

in vec3 var_Position;
in vec3 var_ViewOrigin;

out vec4 frag_color;

vec4 calculateFullyTexturedRoomsColor(
	vec3 position, 
	sampler2D ceilingTexture,
	sampler2D floorTexture,
	sampler2D wallYZTexture,
	sampler2D wallXZTexture,
	vec3 wallFrequencies, 
	vec3 cameraPosition)
{
	//get the vector from camera to surface
	vec3 direction = position - cameraPosition;
	
	//calculate wall locations
	vec3 walls = (floor(position * wallFrequencies) + step(vec3(0, 0, 0), direction)) / wallFrequencies;
	
	//how much of the ray is needed to get from the cameraPosition to each of the walls
	vec3 rayFractions = (vec3(walls.x, walls.y, walls.z) - cameraPosition) / direction;	
	
	//texture-coordinates of intersections
	vec2 intersectionXY = -wallFrequencies.z * (cameraPosition + rayFractions.z * direction).yx;
	vec2 intersectionXZ = -wallFrequencies.y * (cameraPosition + rayFractions.y * direction).xz;
	vec2 intersectionZY = -wallFrequencies.x * (cameraPosition + rayFractions.x * direction).yz;	

	
	//use the intersection as the texture coordinates for the ceiling
	vec4 ceilingColour = texture2D(ceilingTexture, intersectionXY);
	vec4 floorColour = texture2D(floorTexture, intersectionXY);
	vec4 verticalColour = mix(floorColour, ceilingColour, step(0, direction.z));
	vec4 wallXZColour = texture2D(wallXZTexture, intersectionXZ);
	vec4 wallYZColour = texture2D(wallYZTexture, intersectionZY);

	//choose the closest of these walls for the normal for the lighting
	//(some intelligent step-usage is needed here to compensate for the lack of an if-statement in ps_2_0)
	float xVSy = step(rayFractions.x, rayFractions.y);
	vec4 interiorColour = mix(wallXZColour, wallYZColour, xVSy);
	float rayFraction_xVSy = mix(rayFractions.y, rayFractions.x, xVSy);
	float xyVSz = step(rayFraction_xVSy, rayFractions.z);
	interiorColour = mix(verticalColour, interiorColour, xyVSz);
	return interiorColour;
	//return floorColour;
}

void main() {
    
    frag_color = calculateFullyTexturedRoomsColor(
		var_Position.xyz, 
		tex0, tex4, tex2, tex3, 
		vec3(1.0 / 160.0, 1.0 / 160.0, 1.0 / 160.0), 
		var_ViewOrigin.xyz
	);
	
	//frag_color = vec4(var_ViewOrigin.xyz, 1.0);
}
